@@ -1,4 +1,4 @@
-# $Id: Manifest.pm,v 1.14 2005/03/27 02:29:36 comdog Exp $
+# $Id: Manifest.pm 2165 2007-02-22 19:15:19Z comdog $
 package Test::Manifest;
 use strict;
 
@@ -15,13 +15,15 @@ require Test::More;
 @EXPORT    = qw(run_t_manifest);
 @EXPORT_OK = qw(get_t_files make_test_manifest manifest_name);
 
-$VERSION = sprintf "%d.%02d", q$Revision: 1.14 $ =~ m/ (\d+) \. (\d+) /x;
+$VERSION = 1.17; #sprintf "%d.%02d", q$Revision: 2165 $ =~ m/ (\d+) \. (\d+) /x;
 
 my $Manifest = catfile( "t", "test_manifest" );
+my %SeenInclude = ();
+my %SeenTest = ();
 
 require 5.006;
 
-*MY::test_via_harness = sub
+sub MY::test_via_harness
 	{
 	my($self, $perl, $tests) = @_;
 
@@ -127,6 +129,14 @@ argument. You can then define sets of tests and choose a set to
 run. For instance, you might create a set for end users, but also
 add on a set for deeper testing for developers.
 
+Experimentally, you can include a command to grab test names from 
+another file. The command starts with a C<;> to distinguish it
+from a true filename. The filename (currently) is relative to the
+current working directory, unlike the filenames, which are relative
+to C<t/>. The filenames in the included are still relative to C<t/>.
+
+	;include t/file_with_other_test_names.txt
+	
 To select sets of tests, specify the level in the variable TEST_LEVEL
 during `make test`. 
 
@@ -140,19 +150,41 @@ sub get_t_files
 	my $upper_bound = shift;
 	diag( "Test level is $upper_bound\n" ) if $Test::Harness::verbose;
 	
-	carp( "$Manifest does not exist!" ) unless -e $Manifest;
-	return unless open my( $fh ), $Manifest;
+	%SeenInclude = ();
+	%SeenTest    = ();
 
+	carp( "$Manifest does not exist!" ) unless -e $Manifest;
+	my $result = _load_test_manifest($Manifest, $upper_bound);
+	return unless defined $result;
+	my @tests = @{$result};
+	
+	return wantarray ? @tests : join " ", @tests;
+	}
+
+# Wrapper for loading test manifest files to support including other files
+sub _load_test_manifest
+	{
+	my $manifest = shift;
+	return unless open my( $fh ), $manifest;
+
+	my $upper_bound = shift || 0;
 	my @tests = ();
 
 	while( <$fh> )
 		{
-		chomp;
-		s/#.*//;
-		s/^\s+|\s+$//g;
+		s/#.*//; s/^\s+//; s/\s+$//;
+
 		next unless $_;
-		my( $test, $level ) = split /\s+/, $_, 2;
-		
+
+		my( $command, $arg ) = split/\s+/, $_, 2;
+		if( $command eq ';include' ) 
+			{
+			my $result = _include_file( $arg, $., $upper_bound );
+			push @tests, @$result if defined $result;
+			next;
+			}
+
+		my( $test, $level ) = ( $command, $arg );
 		$level = 1 unless defined $level;
 		
 		next if( $upper_bound and $level > $upper_bound );
@@ -162,13 +194,44 @@ sub get_t_files
 			unless $level =~ m/^\d+(?:.\d+)?$/;
 		carp( "test file begins with t/ [$test]" ) if m|^t/|;
 		
-		push @tests, catfile( "t", $test ) if -e catfile( "t", $test );
-		}
-	close $fh;
+		$test = catfile( "t", $test ) if -e catfile( "t", $test );
+		# Make sure we don't include a test we've already seen
+		next if exists $SeenTest{$test};
 
-	return wantarray ? @tests : join " ", @tests;
+		$SeenTest{$test} = 1;
+		push @tests, $test;
+		}
+
+	close $fh;
+	return \@tests;
 	}
 
+sub _include_file
+	{
+	my( $file, $line, $upper_bound ) = @_;
+	diag( "Including file $file at line $line\n" ) if $Test::Harness::verbose;
+	
+	unless( -e $file )
+		{
+		carp( "$file does not exist" ) ;
+		return;
+		}
+	
+	if( exists $SeenInclude{$file} )
+		{
+		carp( "$file already loaded - skipping" ) ;
+		return;
+		}
+
+	$SeenInclude{$file} = $line;
+
+	my $result = _load_test_manifest( $file, $upper_bound );
+	return unless defined $result;
+	
+	$result;
+	}
+	
+	
 =item make_test_manifest()
 
 Creates the test_manifest file in the t directory by reading
@@ -220,17 +283,23 @@ latest sources in CVS, as well as all of the previous releases.
 If, for some reason, I disappear from the world, one of the other
 members of the project can shepherd this module appropriately.
 
+=head1 CREDITS
+
+Matt Vanderpol suggested and supplied a patch for the ;include
+feature.
+
 =head1 AUTHOR
 
 brian d foy, C<< <bdfoy@cpan.org> >>
 
-=head1 COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2002-2004, brian d foy, All Rights Reserved
+Copyright (c) 2002-2007 brian d foy.  All rights reserved.
 
-You may use and distribute this module under the same terms
-as Perl itself
+This program is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
 
 =cut
 
+
 1;
